/*
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER, TCSDESIGNER
 * Version: 1.0
 */


using System;
using System.Collections;
using System.Collections.Specialized;
using System.IO;
using TopCoder.Util.ConfigurationManager;

namespace TopCoder.File.Statistics
{
    /// <summary>
    /// This is the main class of the user API. It provides the basic API, which includes maintaining a list of file
    /// processors, a reporting handler and controls the way to process a file. Files are processed one by one and the
    /// results are aggregated into the reporting. If you want to start a new reporting you should either explicitly
    /// set the reporting or retrieve the old reporting and subsequently reset it.
    ///
    /// There are two ways to process the file:
    /// <list>
    /// <item>Process it with the first matching processor.</item>
    /// <item>Process it with each martching processor.</item>
    /// <item>Refer to ExtendedFileStatistics for advanced usage.</item>
    /// </list>
    /// </summary>
    /// <remarks>
    /// This class can be intialized from a configuration namespace which should be preloaded. It will look at the
    /// following properties:
    /// <list>
    /// <item>
    /// "processors" as a container for file processors. It will contain a list of other processor properties, where
    /// each sub-property should contain a "classname" that specifies the fully qualified class name of an
    /// <c>IFileProcessor</c> implementation which accepts a namespace string and a property prefix as constructor
    /// arguments. This property is required.
    /// </item>
    ///
    /// <item>
    /// "reporting" which contains a "classname" ("reporting.classname") that specifies the fully qualified class name
    /// of an <c>IStatsReporting</c> implementation which accepts a namespace string and a property prefix as
    /// constructor arguments. This property is optional (<c>XmlStatsReporting</c> is the default value).
    /// </item>
    ///
    /// <item>
    /// "matchall" which is a boolean flag that indicates whether all matching processors will be used to process the
    /// file. It is regarded true when the value is either "on", "yes", "true" case-insensitively. This property is
    /// optional.
    /// </item>
    /// </list>
    ///
    /// This class is not thread-safe in that it provides a number of manipulation methods in a non-synchronized
    /// manner. Calling from different threads at separated time will mix the result into the same reporting, which may
    /// not be the desirable behavior.
    /// </remarks>
    public class FileStatistics
    {
        /// <summary>
        /// The property name used to initialize file processors. This property contains a list of other property name
        /// prefixes, suffixed by ".classname" to indicate the fully qualified classname.
        /// </summary>
        public const string ProcessorsPropertyKey = "processors.names";

        /// <summary>
        /// The specific property name of the fully qualified class name of a <c>IStatsReporting</c> implementation
        /// with a constructor taking a namespace string and a prefix string.
        /// </summary>
        public const string ReportingPropertyKey = "reporting";

        /// <summary>
        /// This is the name of a boolean flag property that indicates whether all matching processors will be used to
        /// process the file. It is regarded true when the value is either "on", "yes", "true" case-insensitively. This
        /// property is optional.
        /// </summary>
        public const string MatchAllPropertyKey = "matchall";

        /// <summary>
        /// Appended to individual processor property prefixes to find the fully qualified class name. That class must
        /// implement <c>IFileProcessor</c> and a two-parameter constructor taking a Configuration Manager namespace
        /// and property prefix. (The prefix is equivalent to the prefix this string is appended to.)
        /// </summary>
        public const string ClassnamePropertyKey = ".classname";

        /// <summary>
        /// This dictionary maps the possible values for "true" for the "matchall" property. Any other value is
        /// considered false.
        /// </summary>
        private static readonly IDictionary TrueValues = Helper.CreateDictionary(3, false);

        /// <summary>
        /// Initializes the case-insensitive set of true values. We can't initialize an <c>IDictionary</c> in any more
        /// direct way.
        /// </summary>
        static FileStatistics()
        {
            TrueValues.Add("true", null);
            TrueValues.Add("on", null);
            TrueValues.Add("yes", null);
        }

        /// <summary>
        /// All the processors. It is initalized to a <c>ListDictionary</c>. It's readonly, that is, the reference
        /// itself will never change. The list dictionary is used to iterate in the pre-defined order.
        /// </summary>
        private readonly IDictionary processors = new ListDictionary();

        /// <summary>
        /// The reporting to use, this is mutable. It's initialized to <c>XmlReporting</c> if nothing else is specified
        /// in the configuration.
        /// </summary>
        private IStatsReporting reporting = null;

        /// <summary>
        /// A flag which determines whether to process every file with each matching processor.
        /// </summary>
        private bool matchAll = false;

        /// <summary>
        /// Exposes an aggregated list of the registered file processors.
        /// </summary>
        /// <value>
        /// An <c>IList</c> of <c>IFileProcessor</c> instances.
        /// </value>
        public IList AllFileProcessors
        {
            get
            {
                return new ArrayList(processors.Values);
            }
        }

        /// <summary>
        /// Exposes the reporting instance, both getter and setter.
        /// </summary>
        /// <exception cref="ArgumentNullException">In setter, if a null value is specified.</exception>
        /// <value>
        /// <c>reporting</c>, a non-null <c>IStatsReporting</c> instance.
        /// </value>
        public IStatsReporting Reporting
        {
            get
            {
                return reporting;
            }
            set
            {
                if (value == null)
                {
                    throw new ArgumentNullException("value");
                }
                reporting = value;
            }
        }

        /// <summary>
        /// Gets/sets whether each file should be processed by each matching processor (not just the first one).
        /// </summary>
        public bool IsMatchAll
        {
            get
            {
                return matchAll;
            }
            set
            {
                matchAll = value;
            }
        }

        /// <summary>
        /// Retrieves a file processor with a specific alias, or null.
        /// </summary>
        /// <param name="alias">The file processor alias.</param>
        /// <exception cref="ArgumentNullException">If <c>alias</c> is null.</exception>
        /// <exception cref="ArgumentException">If <c>alias</c>, when trimmed, is empty.</exception>
        /// <returns>
        /// The associated <c>IFileProcessor</c> for <c>alias</c>, or null, if no such association exists.
        /// </returns>
        public IFileProcessor this[string alias]
        {
            get
            {
                if (alias == null)
                {
                    throw new ArgumentNullException("alias");
                }
                if (alias.Trim().Length == 0)
                {
                    throw new ArgumentException("The specified string is empty.", "alias");
                }

                return (IFileProcessor) processors[alias];
            }
        }

        /// <summary>
        /// Default constructor, initializing without any configuration.
        /// </summary>
        public FileStatistics()
        {
            reporting = new Reporting.XmlStatsReporting();
        }

        /// <summary>
        /// A constructor loading a configuration from the given, already loaded, namespace. It loads a processor list,
        /// a reporting method and the "matchall" flag.
        /// </summary>
        /// <param name="nameSpace">The namespace to load from.</param>
        /// <exception cref="ArgumentNullException">If <c>namespace</c> is null.</exception>
        /// <exception cref="ArgumentException">If <c>namespace</c> is empty, when trimmed.</exception>
        /// <exception cref="ConfigurationException">If a config error occurs.</exception>
        public FileStatistics(string nameSpace)
        {
            if (nameSpace == null)
            {
                throw new ArgumentNullException("nameSpace");
            }
            if (nameSpace.Trim().Length == 0)
            {
                throw new ArgumentException("Specified string empty when trimmed.", "nameSpace");
            }

            try
            {
                // Load the processor names list
                string[] processors = ConfigManager.GetInstance().GetValues(nameSpace, ProcessorsPropertyKey);

                if (processors == null)
                {
                    throw new ConfigurationException("No list of processor names specified.");
                }

                if (processors.Length == 0)
                {
                    throw new ConfigurationException("Empty processor list specified.");
                }

                foreach (string value in processors)
                {
                    // Find the classname
                    string className = ConfigManager.GetInstance().GetValue(nameSpace, value + ClassnamePropertyKey);
                    if (className == null)
                    {
                        throw new ConfigurationException("Missing classname for listed processor: " + value);
                    }

                    // Create the instance, through the contract constructor, add it
                    AddFileProcessor((IFileProcessor) Activator.CreateInstance(Type.GetType(className, true),
                        new object[] { nameSpace, value }));
                }

                string reportingClass = ConfigManager.GetInstance().GetValue(nameSpace, ReportingPropertyKey +
                    ClassnamePropertyKey);

                if (reportingClass != null)
                {
                    Reporting = (IStatsReporting) Activator.CreateInstance(Type.GetType(reportingClass, true),
                        new object[] { nameSpace, ReportingPropertyKey });
                }
                else
                {
                    // Default reporting method
                    Reporting = new Reporting.XmlStatsReporting();
                }

                string matchAllValue = ConfigManager.GetInstance().GetValue(nameSpace, MatchAllPropertyKey);
                if (matchAllValue != null && TrueValues.Contains(matchAllValue))
                {
                    // If the configuraiton contains a recognized string for "true", according to a case-insensitive
                    // comparison, set it accordingly.
                    matchAll = true;
                }
            }
            catch (ConfigurationException)
            {
                throw;
            }
            catch (Exception e)
            {
                throw new ConfigurationException("Error while loading configuration.", e);
            }
        }

        /// <summary>
        /// Adds a file processor for aggregation. If there is already a processor with the same alias, it is replaced.
        /// </summary>
        /// <param name="processor">The file processor to add.</param>
        /// <exception cref="ArgumentNullException">If <c>processor</c> is null.</exception>
        public void AddFileProcessor(IFileProcessor processor)
        {
            if (processor == null)
            {
                throw new ArgumentNullException("processor");
            }

            // Remove any existing, to get a proper order. This is unnecessarily expensive, but it doesn't matter much.
            // Repeated addition means, however, O(N^2).
            RemoveFileProcessor(processor.Alias);
            processors[processor.Alias] = processor;
        }

        /// <summary>
        /// Removes the file processor with the given alias.
        /// </summary>
        /// <param name="alias">Alias for the processor to remove.</param>
        /// <exception cref="ArgumentNullException">If <c>alias</c> is null.</exception>
        /// <exception cref="ArgumentException">If <c>alias</c>, when trimmed, is empty.</exception>
        /// <returns>
        /// If no such processor exists, null is returned. Otherwise the previous value is returned.
        /// </returns>
        public IFileProcessor RemoveFileProcessor(string alias)
        {
            if (alias == null)
            {
                throw new ArgumentNullException("alias");
            }
            if (alias.Trim().Length == 0)
            {
                throw new ArgumentException("Empty string specified.", "alias");
            }

            IFileProcessor processor = (IFileProcessor) processors[alias];
            processors.Remove(alias);

            // Return the previous value, or null.
            return processor;
        }

        /// <summary>
        /// Clears all registered file processors.
        /// </summary>
        public void ClearAllFileProcessors()
        {
            processors.Clear();
        }

        /// <summary>
        /// Processes a single file. This means finding the first matching processor if <c>matchAll</c> is false, or
        /// using all of them if <c>matchAll</c> is true. The file is processed with this/these processor(s).
        /// </summary>
        /// <remarks>
        /// If <c>matchAll</c> is false, the processors are tested in a "first added" manner.
        /// </remarks>
        /// <param name="file">The file to process.</param>
        /// <exception cref="ArgumentNullException">If <c>file</c> is null.</exception>
        /// <exception cref="IOException">If any IO operation fails.</exception>
        /// <exception cref="FileStatisticsException">If processing fails.</exception>
        public void ProcessFile(FileInfo file)
        {
            if (file == null)
            {
                throw new ArgumentNullException("file");
            }

            foreach (DictionaryEntry entry in processors)
            {
                IFileProcessor processor = (IFileProcessor) entry.Value;
                // Does the file match this processor?
                if (processor.FileMatcher.Match(file))
                {
                    reporting.Aggregate(processor, file, processor.Process(file));

                    // If we shouldn't match all, we quit after the first sucessful processing.
                    if (!matchAll)
                    {
                        return;
                    }
                }
            }
        }
    }
}