/*
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER, TCSDESIGNER
 * Version: 1.0
 */

using System;
using System.IO;
using TopCoder.Unix.MagicNumbers;

namespace TopCoder.File.Statistics.Matcher
{
    /// <summary>
    /// An abstract implementation of <c>IFileMatcher</c> that classifies files by file type. File type is determined
    /// by the Magic Number component and its configuration.
    /// </summary>
    /// <remarks>
    /// No configuration is read by this class. The implementation is stateless and thread-safe, noting that it is
    /// dependent on loading the Magic Number component and its configuration. Subclasses may break either or both of
    /// these (stateless, thread-safe), but they are recommended not to. Subclasses will probably load a configuration.
    /// </remarks>
    public abstract class FileTypeMatcher : IFileMatcher
    {
        /// <summary>
        /// A shared instance of the magic number
        /// </summary>
        private MagicNumber magicNumbers;

        /// <summary>
        /// Default constructor. Initializes an instance of the Magic Number component.
        /// </summary>
        protected FileTypeMatcher()
        {
            magicNumbers = new MagicNumber();
        }

        /// <summary>
        /// Extracts the file type and passes the call on to the specific name checking implementation of the subclass.
        /// </summary>
        /// <remarks>
        /// The type is extracted by the Magic Numbers component.
        /// </remarks>
        /// <param name="file">The file to match.</param>
        /// <exception cref="ArgumentNullException">If <c>file</c> is null.</exception>
        /// <exception cref="FileMatchingException">Other error conditions.</exception>
        /// <returns><c>true</c> if the file name matches, <c>false</c> otherwise.</returns>
        public bool Match(FileInfo file)
        {
            if (file == null)
            {
                throw new ArgumentNullException("file");
            }

            FileType type;
            try
            {
                type = magicNumbers.DetermineFileType(file);
            }
            catch (IOException)
            {
                throw;
            }
            catch (Exception e)
            {
                throw new FileMatchingException("Error identifying file type.", e);
            }

            // If not identified, it can't match, according to design.
            if (type == null)
            {
                return false;
            }

            return Match(type);
        }

        /// <summary>
        /// Template method to match a file depending on its type, determined by the Magic Number component.
        /// </summary>
        /// <param name="type">The file type.</param>
        /// <exception cref="FileMatchingException">Any error condition.</exception>
        /// <returns>True if the file is matched, false otherwise.</returns>
        protected abstract bool Match(FileType type);
    }
}