/*
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER, TCSDESIGNER
 * Version: 1.0
 */

using System;
using System.IO;
using System.Collections;
using System.Collections.Specialized;

namespace TopCoder.File.Statistics.Reporting
{
    /// <summary>
    /// This class provides a <c>IStatsReporting</c> implementation base. It aggregates results from each processor and
    /// classifies them based on single file, directory or alias. It also provides an overall stats. It defers the
    /// actual rendering logic to concrete implementations (<c>BasicStatsReporting</c> and <c>XmlStatsReporting</c>).
    /// </summary>
    /// <remarks>
    /// This class has a constructor with a namespace and a property for extension purposes. There is no real
    /// configuration parsing logic there at this point in time.
    /// </remarks>
    public class BasicStatsReporting : AbstractStatsReporting
    {
        /// <summary>
        /// The default indent used in reports.
        /// </summary>
        /// <value>
        /// "    " (four spaces)
        /// </value>
        private const string DefaultIndent = "    ";

        /// <summary>
        /// A very specific string format specifier for long formatting. It's used twice, first as a test, to get the
        /// needed spacing to achieve right-alignment.
        /// </summary>
        private const string NumericFormatSpecifier = ":N0";

        /// <summary>
        /// String used when there are no metrics.
        /// </summary>
        private const string NoData = "No statistic data available for this item.";

        /// <summary>
        /// Default constructor. Does nothing.
        /// </summary>
        public BasicStatsReporting() : base()
        {
        }

        /// <summary>
        /// Constructor taking a reference to a configuration manager property. Mainly provided for future
        /// extensibility. In this release, the parameters are only checked superficially.
        /// </summary>
        /// <param name="nameSpace">The configuration manager namespace.</param>
        /// <param name="prefix">The prefix to the properties for this reporting method.</param>
        /// <exception cref="ArgumentNullException">If <c>nameSpace</c> or <c>prefix</c> is null.</exception>
        /// <exception cref="ConfigurationException">If any config error occurs.</exception>
        public BasicStatsReporting(string nameSpace, string prefix) : base(nameSpace, prefix)
        {
        }

        /// <summary>
        /// Creates a report section for a specific collection, with a header if there are multiple elements.
        /// </summary>
        /// <param name="writer">The target for writing the report.</param>
        /// <param name="single">The actual item type, for a single instance.</param>
        /// <param name="multiple">The heading for multiple instances.</param>
        /// <param name="stats">The stats collection to report on.</param>
        private void ReportCollection(TextWriter writer, string single, string multiple, StatsCollection stats)
        {
            // We need an ArrayList here for convenient sorting.
            ArrayList list = new ArrayList(stats.AllKeys);
            list.Sort();

            string indent = "";
            switch (list.Count)
            {
                case 0: // If empty, we do nothing
                    return;
                case 1: // If there is only one entry, we just write it directly
                    indent = "";
                    break;
                default: // If there are multiple entries, we first write a summary, and the later items are indented
                    indent = DefaultIndent;
                    writer.WriteLine(multiple);
                    foreach (string str in list)
                    {
                        // This indention is independent of the item indent above, but it should be deeper.
                        writer.WriteLine("{1}{2}{0}", str, DefaultIndent, DefaultIndent);
                    }
                    writer.WriteLine();
                    break;
            }

            // Now, write each individual item
            foreach (string str in list)
            {
                ReportSpecific(writer, string.Format("{0} {1}:", single, str), stats[str], indent);
            }

            // Some spacing between categories
            writer.WriteLine("--");
            writer.WriteLine();
        }

        /// <summary>
        /// Reports a single collection of statistic results.
        /// </summary>
        /// <param name="writer">The target writer.</param>
        /// <param name="header">The header line for this.</param>
        /// <param name="result">The result to report.</param>
        /// <param name="indent">The indent to prepend for each line.</param>
        private void ReportSpecific(TextWriter writer, string header, StatsResult result, string indent)
        {
            IList metrics = result.Metrics;

            writer.WriteLine("{0}{1}", indent, header);
            if (metrics.Count == 0)
            {
                // Indent further, report that there is nothing
                writer.WriteLine("{0}{1}{2}", indent, DefaultIndent, NoData);
            }
            else
            {
                // Create a list of all metrics, with names, for sorting
                // We don't use an IDictionary entry, as the "keys" might coincide
                ArrayList sorted = new ArrayList(metrics.Count);

                // We want every : lined up to the right in the same report section
                int maxLength = 0;
                int maxNumLength = 0;
                string testNumericFormat = "{0" + NumericFormatSpecifier + "}";

                foreach (Metric metric in metrics)
                {
                    sorted.Add(new DictionaryEntry(metric.Name, metric));
                    maxLength = Math.Max(maxLength, metric.Name.Length);
                    maxNumLength = Math.Max(maxNumLength, string.Format(testNumericFormat, result[metric]).Length);
                }

                // Now we really sort it
                sorted.Sort();
                // Minimum two spaces
                maxLength += 2;
                maxLength /= 4;
                maxLength++;

                // Get an "even" spacing. We don't use DefaultIndent here, as that may be changed to a tab character.
                maxLength *= 4;

                // A format string created dynamically with the desired field width for the string field.
                string formatString = "{0}{1}{2,-" + maxLength + "}: " +
                    "{3," + maxNumLength + NumericFormatSpecifier + "}";

                foreach (DictionaryEntry entry in sorted)
                {
                    // Now we can print it out in sorted order
                    Metric metric = (Metric) entry.Value;
                    // Format each individual line
                    writer.WriteLine(formatString, indent, DefaultIndent, entry.Key, result[metric]);
                }
            }
            // Some vertical spacing to the next item.
            writer.WriteLine();
        }


        /// <summary>
        /// Generate a report with all the aggregated results.
        /// </summary>
        /// <exception cref="StatsReportingException">If any error occurs.</exception>
        /// <returns>A string containing the complete generated report.</returns>
        public override string GenerateReport()
        {
            try
            {
                StringWriter writer = new StringWriter();
                writer.WriteLine("File statistics report");
                writer.WriteLine();
                ReportCollection(writer, "File processor", "File processor aliases:", AliasStats);
                ReportCollection(writer, "Directory", "Directories:", DirectoryStats);
                ReportCollection(writer, "File", "Files:", FileStats);
                ReportSpecific(writer, "Overall results:", OverallStats, "");

                return writer.ToString();
            }
            catch (Exception e)
            {
                throw new StatsReportingException("Reporting error.", e);
            }
        }
    }
}