/*
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER, TCSDESIGNER
 * Version: 1.0
 */

using System;
using System.IO;
using System.Collections;
using System.Collections.Specialized;
using System.Xml;
using System.Xml.Xsl;
using TopCoder.Util.ConfigurationManager;

namespace TopCoder.File.Statistics.Reporting
{
    /// <summary>
    /// XmlStatsReporting is a reporting implementation that renders XML reports. Refer to component specification for
    /// formatting of the XML. It is capable of using a stylesheet to transform the XML into user-oriented format
    /// (HTML, CSV, etc.)
    /// </summary>
    /// <remarks>
    /// This class has a constructor with a namespace and a property for extension purposes. It supports a single
    /// property "stylesheet" which contains a filename of the stylesheet. This property is optional. If it's missing
    /// raw XML is generated.
    ///
    /// This class is not thread-safe.
    /// </remarks>
    public class XmlStatsReporting : AbstractStatsReporting
    {
        /// <summary>
        /// The subproperty which specifies a stylesheet to apply on the xml. If this property is not specified, the
        /// raw output is generated.
        /// </summary>
        public const string StylesheetPropertyKey = ".stylesheet";

        /// <summary>
        /// The root element of all reports.
        /// </summary>
        private const string ReportElementName = "report";

        /// <summary>
        /// The name of the XML element containing specific alias results. A name attribute is also present.
        /// </summary>
        private const string AliasElementName = "alias";

        /// <summary>
        /// The name of the XML element containing specific file results. A name attribute is also present.
        /// </summary>
        private const string FileElementName = "file";

        /// <summary>
        /// The name of the XML element containing specific directory results. A name attribute is also present.
        /// </summary>
        private const string DirectoryElementName = "directory";

        /// <summary>
        /// The name of the XML element containing all specific alias elements underneath.
        /// </summary>
        private const string AliasesElementName = "aliases";

        /// <summary>
        /// The name of the XML element containing all specific file elements underneath.
        /// </summary>
        private const string FilesElementName = "files";

        /// <summary>
        /// The name of the XML element containing all specific directory elements underneath.
        /// </summary>
        private const string DirectoriesElementName = "directories";

        /// <summary>
        /// The element name for the overall stats, the aggregated data for all files surveyed.
        /// </summary>
        private const string OverallElementName = "overall";

        /// <summary>
        /// The attribute name used to specify the name for individual alias, directory, and file elements.
        /// </summary>
        private const string NameAttributeName = "name";

        /// <summary>
        /// Stores a reference to the XSLT stylesheet used, if any.
        /// </summary>
        private XslTransform stylesheet = null;

        /// <summary>
        /// Default constructor. Does nothing.
        /// </summary>
        public XmlStatsReporting() : base()
        {
        }

        /// <summary>
        /// Constructor taking a reference to a configuration manager property. A XSL stylesheet can be provided under
        /// the "subproperty" of ".stylesheet".
        /// </summary>
        /// <param name="nameSpace">The configuration manager namespace.</param>
        /// <param name="prefix">The prefix to the properties for this reporting method.</param>
        /// <exception cref="ArgumentNullException">If <c>nameSpace</c> or <c>prefix</c> is null.</exception>
        /// <exception cref="ConfigurationException">If any config error occurs.</exception>
        public XmlStatsReporting(string nameSpace, string prefix)
            : base(nameSpace, prefix)
        {
            try
            {
                string configValue = ConfigManager.GetInstance().GetValue(nameSpace, prefix + StylesheetPropertyKey);

                // Now check the optional value,
                if (configValue != null)
                {
                    stylesheet = new XslTransform();
                    stylesheet.Load(configValue);
                }
            }
            catch (Exception e)
            {
                throw new ConfigurationException("Configuration error.", e);
            }
        }

        /// <summary>
        /// Constructor taking a file reference to a XSL stylesheet.
        /// </summary>
        /// <param name="file">The XSL stylesheet.</param>
        /// <exception cref="ArgumentNullException">If <c>stylesheet</c> is null.</exception>
        /// <exception cref="ArgumentException">If <c>stylesheet</c> isn't a valid file.</exception>
        public XmlStatsReporting(FileInfo file)
        {
            if (file == null)
            {
                throw new ArgumentNullException("file");
            }

            try
            {
                stylesheet = new XslTransform();
                stylesheet.Load(file.FullName);
            }
            catch (Exception e)
            {
                throw new ArgumentException("Stylesheet error.", e);
            }
        }

        /// <summary>
        /// Creates a report section for a specific collection, with a header if there are multiple elements.
        /// </summary>
        /// <param name="node">The node where the element and the subelements should be added.</param>
        /// <param name="singleElement">The element name for the individual items.</param>
        /// <param name="groupElement">The root element name, containing item elements.</param>
        /// <param name="stats">The stats collection to report on.</param>
        private void ReportCollection(XmlNode node, string singleElement, string groupElement, StatsCollection stats)
        {
            XmlNode element = node.OwnerDocument.CreateElement(groupElement);
            node.AppendChild(element);

            // We need an ArrayList here for convenient sorting.
            ArrayList list = new ArrayList(stats.AllKeys);
            list.Sort();

            // Now, write each individual item
            foreach (string str in list)
            {
                ReportSpecific(element, singleElement, str, stats[str]);
            }
        }

        /// <summary>
        /// Reports a single collection of statistic results.
        /// </summary>
        /// <param name="node">The node where to add this XML element.</param>
        /// <param name="elementName">The XML element to use.</param>
        /// <param name="name">The name attribute to use in the element, omitted if null.</param>
        /// <param name="result">The result to report.</param>
        private void ReportSpecific(XmlNode node, string elementName, string name, StatsResult result)
        {
            XmlElement element = node.OwnerDocument.CreateElement(elementName);
            node.AppendChild(element);

            // Was a name specified?
            if (name != null)
            {
                XmlAttribute nameAttribute = element.OwnerDocument.CreateAttribute(NameAttributeName);
                nameAttribute.Value = name;

                element.Attributes.Append(nameAttribute);
            }

            // Retrieve a list of all metrics
            IList metrics = result.Metrics;


            // Create a list of all metrics, with names, for sorting
            // We don't use an IDictionary entry, as the "keys" might coincide
            ArrayList sorted = new ArrayList(metrics.Count);

            // We want every : lined up to the right in the same report section
            foreach (Metric metric in metrics)
            {
                sorted.Add(new DictionaryEntry(metric.DisplayName, metric));
            }

            // Now we really sort it
            sorted.Sort();

            foreach (DictionaryEntry entry in sorted)
            {
                // Now we can print it out in sorted order
                Metric metric = (Metric) entry.Value;
                XmlNode metricElement = node.OwnerDocument.CreateElement((string) entry.Key);

                // Default ToString conversion for long to XML is ok
                metricElement.AppendChild(node.OwnerDocument.CreateTextNode(result[metric].ToString()));

                element.AppendChild(metricElement);
            }
        }


        /// <summary>
        /// Generate a report with all the aggregated results.
        /// </summary>
        /// <exception cref="StatsReportingException">If any error occurs.</exception>
        /// <returns>A string containing the complete generated report.</returns>
        public override string GenerateReport()
        {
            XmlDocument document = new XmlDocument();
            XmlNode node = document.CreateElement(ReportElementName);
            document.AppendChild(node);
            ReportCollection(node, FileElementName, FilesElementName, FileStats);
            ReportCollection(node, DirectoryElementName, DirectoriesElementName, DirectoryStats);
            ReportCollection(node, AliasElementName, AliasesElementName, AliasStats);
            ReportSpecific(node, OverallElementName, null, OverallStats);


            // Transform?
            if (stylesheet != null)
            {
                StringWriter writer = new StringWriter();

                XmlTextWriter xmlWriter = new XmlTextWriter(writer);
                stylesheet.Transform(document, new XsltArgumentList(), xmlWriter, new XmlUrlResolver());

                return writer.ToString();
            }
            else
            {
                StringWriter writer = new StringWriter();
                document.Save(writer);

                return writer.ToString();
            }
        }
    }
}