/*
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER, TCSDESIGNER
 * Version: 1.0
 */

using System;
using System.Reflection;
using TopCoder.Util.ConfigurationManager;
using NUnit.Framework;
using System.IO;

namespace TopCoder.File.Statistics.Reporting
{
    /// <summary>
    /// This class is a base for the tests of the two reporting methods. They share most their logic, so this base
    /// class is highly convenient. In addition, this class benefits to a degree from the <c>TestContract</c> class.
    /// </summary>
    [TestFixture]
    public abstract class TestAbstractStatsReporting : TestContract
    {
        /// <summary>
        /// Used during testing.
        /// </summary>
        protected AbstractStatsReporting[] instances = new AbstractStatsReporting[2];

        /// <summary>
        /// The processor used in our tiny tests.
        /// </summary>
        protected IFileProcessor processor =
            new Processor.LineCounter.TextLineCounter("alias", new Matcher.AnyFileMatcher());

        /// <summary>
        /// The base directory used for one of the instances.
        /// </summary>
        protected DirectoryInfo baseDir = new DirectoryInfo(@"..\..\test_files");

        /// <summary>
        /// Aggregates the specified file.
        /// </summary>
        /// <param name="index">The instance index.</param>
        /// <param name="file">The file to aggregate.</param>
        public void AggregateFile(int index, FileInfo file)
        {
            instances[index].Aggregate(processor, file, processor.Process(file));
        }

        /// <summary>
        /// Create an instance used for testing.
        /// </summary>
        [SetUp]
        public virtual void Setup()
        {
            for (int i = 0; i < 2; i++)
            {
                instances[i] = (AbstractStatsReporting) Constructor();
            }

            // One is reset, the other not, so we can make some comparisons
            instances[0].Reset(baseDir);

            for (int i = 0; i < 2; i++)
            {
                AggregateFile(i, new FileInfo(@"..\..\test_files\sub\TestFile1.cs"));
                AggregateFile(i, new FileInfo(@"..\..\test_files\sub\exe\Test.exe"));
            }
        }

        /// <summary>
        /// Checks out the file stats.
        /// </summary>
        [Test]
        public void TestFileStats()
        {
            for (int i = 0; i < 2; i++)
            {
                Assert.AreEqual(2, instances[i].FileStats.AllKeys.Count, "Two files were surveyed.");
            }
            Metric metric = new Metric("Line Count", "a", "b");

            Assert.AreEqual(8, instances[0].FileStats[@"sub\TestFile1.cs"][metric],
                "Line count expected for cs file.");
            Assert.AreEqual(204, instances[0].FileStats[@"sub\exe\test.exe"][metric],
                "Line count expected for exe file.");
        }

        /// <summary>
        /// Checks out the directory stats.
        /// </summary>
        [Test]
        public void TestDirectoryStats()
        {
            for (int i = 0; i < 2; i++)
            {
                Assert.IsTrue(2 + i <= instances[i].DirectoryStats.AllKeys.Count, "At least two dirs were surveyed.");
            }
            Metric metric = new Metric("Line Count", "a", "b");

            Assert.AreEqual(212, instances[0].DirectoryStats[@"sub"][metric],
                "Line count expected for sub dir.");
            Assert.AreEqual(204, instances[0].DirectoryStats[@"sub\exe"][metric],
                "Line count expected for sub\\exe dir.");
        }

        /// <summary>
        /// Checks out the alias stats.
        /// </summary>
        [Test]
        public void TestAliasStats()
        {
            Metric metric = new Metric("Line Count", "a", "b");

            for (int i = 0; i < 2; i++)
            {
                Assert.AreEqual(212, instances[i].AliasStats[@"alias"][metric],
                    "Line count expected for alias 'alias'.");
            }
        }

        /// <summary>
        /// Checks out the overall stats.
        /// </summary>
        [Test]
        public void TestOverallStats()
        {
            Metric metric = new Metric("Line Count", "a", "b");

            for (int i = 0; i < 2; i++)
            {
                Assert.AreEqual(212, instances[i].OverallStats[metric],
                    "Line count expected for overall stats.");
            }
        }
    }
}