/*
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER, TCSDESIGNER
 * Version: 1.0
 */

using System;
using System.Reflection;
using TopCoder.Util.ConfigurationManager;
using NUnit.Framework;

namespace TopCoder.File.Statistics.Reporting
{
    /// <summary>
    /// This class is used to test the stats collection class. This is a key mapping with automatic aggregation of
    /// <c>StatsResult</c> instances.
    /// </summary>
    [TestFixture]
    public class TestStatsCollection
    {
        /// <summary>
        /// This instance is used during testing.
        /// </summary>
        private StatsCollection instance;

        /// <summary>
        /// Three metrics used for testing.
        /// </summary>
        private Metric[] metrics = new Metric[3];

        /// <summary>
        /// Three <c>StatsResult</c> instances used for testing.
        /// </summary>
        private StatsResult[] results = new StatsResult[3];

        /// <summary>
        /// Sets up some simple objects used for testing.
        /// </summary>
        [SetUp]
        public void Setup()
        {
            metrics[0] = new Metric("A", "B", "C");
            metrics[1] = new Metric("A", "C", "B");
            metrics[2] = new Metric("D", "B", "C");

            results[0] = new StatsResult();
            results[1] = new StatsResult();
            results[2] = new StatsResult();

            results[1][metrics[0]] = 1;
            // When we merge result 1 and 2, metrics 0 and 1 should be aggregated, as they share the name "A"
            results[2][metrics[1]] = 2;
            results[2][metrics[2]] = 4;

            instance = new StatsCollection();
            instance.Add("one", results[0]);
            instance.Add("one", results[1]);
            instance.Add("one", results[2]);
            instance.Add("two", results[2]);
        }

        /// <summary>
        /// Tests clearing the stats collection.
        /// </summary>
        [Test]
        public void TestClear()
        {
            instance.Clear();
            Assert.AreEqual(0, instance.AllKeys.Count, "No keys should be present after clearing.");
        }

        /// <summary>
        /// Tests the <c>Add</c> method with null params.
        /// </summary>
        [ExpectedException(typeof(ArgumentNullException))]
        [Test]
        public void TestAddNull1()
        {
            instance.Add(null, results[1]);
        }

        /// <summary>
        /// Tests the <c>Add</c> method with an empty key.
        /// </summary>
        [ExpectedException(typeof(ArgumentException))]
        [Test]
        public void TestAddEmpty()
        {
            instance.Add(" ", results[1]);
        }

        /// <summary>
        /// Tests the <c>Add</c> method with null params.
        /// </summary>
        [ExpectedException(typeof(ArgumentNullException))]
        [Test]
        public void TestAddNull2()
        {
            instance.Add("three", null);
        }

        /// <summary>
        /// Tests adding to a key that so far hasn't been in the collection.
        /// </summary>
        [Test]
        public void TestAddNew()
        {
            Assert.IsFalse(instance.AllKeys.Contains("three"), "Key three shouldn't be pre-existing.");
            instance.Add("three", results[2]);
            Assert.AreEqual(results[2][metrics[1]], instance["three"][metrics[1]],
                "The value should be equivalent to the one from results 2. 1");
            Assert.AreEqual(results[2][metrics[2]], instance["three"][metrics[2]],
                "The value should be equivalent to the one from results 2. 2");
        }

        /// <summary>
        /// Tests adding to a key that already exists, utilising the point of the class, aggregation.
        /// </summary>
        [Test]
        public void TestAddExisting()
        {
            instance.Add("one", results[2]);
            Assert.AreEqual(5, instance["one"][metrics[1]],
                "The value should be a sum. 1");
            Assert.AreEqual(8, instance["one"][metrics[2]],
                "The value should be a sum. 2");
        }

        /// <summary>
        /// Tests the indexer method.
        /// </summary>
        [Test]
        public void TestIndexer()
        {
            Assert.IsNull(instance["three"], "Key three shouldn't exist.");
            Assert.IsFalse(instance["two"] == instance["two"], "A new clone should be returned each time.");
        }

        /// <summary>
        /// Tests a null parameter.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestIndexerNull()
        {
            StatsResult res = instance[null];
        }

        /// <summary>
        /// Tests an empty parameter.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestIndexerEmpty()
        {
            StatsResult res = instance["     "];
        }
    }
}