/*
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER, TCSDESIGNER
 * Version: 1.0
 */

using System;
using System.IO;
using System.Reflection;
using TopCoder.Util.ConfigurationManager;
using NUnit.Framework;

namespace TopCoder.File.Statistics
{
    /// <summary>
    /// This class tests the operation of <c>FileStatistics</c>, mainly based on the assumption that the underlying
    /// classes, which are tested through other, individual, tests, work properly.
    /// </summary>
    [TestFixture]
    public class TestFileStatistics
    {
        /// <summary>
        /// This field is set in subclasses for proper instantiation.
        /// </summary>
        protected Type type = typeof(FileStatistics);

        /// <summary>
        /// The instances used for testing, one from config, one "clean".
        /// </summary>
        protected FileStatistics[] instances = new FileStatistics[2];

        /// <summary>
        /// Sets up the testing instances.
        /// </summary>
        [SetUp]
        public virtual void Setup()
        {
            instances[0] = Constructor();
            instances[1] = Constructor("TopCoder.File.Statistics.FileStatistics");
        }

        /// <summary>
        /// Sets up the testing fixture.
        /// </summary>
        [TestFixtureSetUp]
        public void FixtureSetup()
        {
            ConfigManager.GetInstance().Clear(false);
            ConfigManager.GetInstance().LoadFile(@"..\..\test_files\test.xml");
        }

        /// <summary>
        /// Invokes the class constructor.
        /// </summary>
        /// <param name="args">The desired constructor parameters.</param>
        /// <returns>
        /// An object.
        /// </returns>
        public FileStatistics Constructor(params object[] args)
        {
            try
            {
                return (FileStatistics) Activator.CreateInstance(type, args);
            }
            catch (TargetInvocationException e)
            {
                // Expose the inner exception for normal test fixture operation
                throw e.InnerException;
            }
        }

        /// <summary>
        /// Tests the <c>AllFileProcessors</c> property.
        /// </summary>
        [Test]
        public void TestAllFileProcessors()
        {
            Assert.AreEqual(0, instances[0].AllFileProcessors.Count, "The empty object should be empty.");
            Assert.AreEqual(2, instances[1].AllFileProcessors.Count,
                "Two processors should be present in the from-config object");

            bool anyText = false;
            bool anyCStyle = false;

            for (int i = 0; i < 2; i++)
            {
                anyText |= (instances[1].AllFileProcessors[i].GetType().
                    Equals(typeof(Processor.LineCounter.TextLineCounter)));
                anyCStyle |= (instances[1].AllFileProcessors[i].GetType().
                    Equals(typeof(Processor.LineCounter.CStyleLineCounter)));
            }

            Assert.IsTrue(anyText && anyCStyle, "Both kinds of processors should be present.");
        }

        /// <summary>
        /// Tests the <c>Reporting</c> property, including setting it.
        /// </summary>
        [Test]
        public void TestReporting()
        {
            Assert.AreEqual(typeof(Reporting.XmlStatsReporting), instances[0].Reporting.GetType(),
                "XML is the default reporter.");
            Assert.AreEqual(typeof(Reporting.BasicStatsReporting), instances[1].Reporting.GetType(),
                "Basic is the configured reporter.");

            instances[1].Reporting = instances[0].Reporting;
            Assert.AreEqual(typeof(Reporting.XmlStatsReporting), instances[1].Reporting.GetType(),
                "Reporter changed for second object.");
        }

        /// <summary>
        /// Tests setting the repoter to null.
        /// </summary>
        [ExpectedException(typeof(ArgumentNullException))]
        [Test]
        public void TestSetReportingNull()
        {
            instances[1].Reporting = null;
        }

        /// <summary>
        /// Tests the match all property.
        /// </summary>
        [Test]
        public void TestMatchAll()
        {
            Assert.IsFalse(instances[0].IsMatchAll, "Match all off by default.");
            Assert.IsTrue(instances[1].IsMatchAll, "Match all configured to be on.");

            instances[0].IsMatchAll = true;
            Assert.IsTrue(instances[0].IsMatchAll, "Match all turned on.");
        }

        /// <summary>
        /// Tests the one-param constructor with a null namespace.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructorNull()
        {
            Constructor(new object[] { null } );
        }

        /// <summary>
        /// Tests the one-param constructor with an empty namespace string.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestConstructorEmptyParam()
        {
            Constructor(" ");
        }

        /// <summary>
        /// Tests the one-param constructor with an unavailable namespace.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ConfigurationException))]
        public void TestConstructorFakeNamespace()
        {
            Constructor("idontexist");
        }

        /// <summary>
        /// Tests the one-param constructor with a namespace that exists, but is void of properties.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ConfigurationException))]
        public void TestConstructorEmptyNamespace()
        {
            Constructor("empty");
        }

        /// <summary>
        /// Tests removing a file processor.
        /// </summary>
        [Test]
        public void TestRemoveFileProcessor()
        {
            Assert.IsNull(instances[0].RemoveFileProcessor("cs"), "Removing a processor that didn't exist.");
            Assert.AreEqual(0, instances[0].AllFileProcessors.Count, "Count should still be 0.");

            Assert.AreEqual(typeof(Processor.LineCounter.CStyleLineCounter),
                instances[1].RemoveFileProcessor("cs").GetType(), "Removing a processor that does exist.");
            Assert.AreEqual(1, instances[1].AllFileProcessors.Count, "Count should be decremented.");
        }

        /// <summary>
        /// Tests removing a null file processor.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestRemoveFileProcessorNull()
        {
            instances[0].RemoveFileProcessor(null);
        }

        /// <summary>
        /// Tests removing a file processor with an empty string alias.
        /// </summary>
        [ExpectedException(typeof(ArgumentException))]
        [Test]
        public void TestRemoveFileProcessorEmpty()
        {
            instances[0].RemoveFileProcessor(" ");
        }

        /// <summary>
        /// Tests adding an existing file processor alias. This works.
        /// </summary>
        [Test]
        public void TestAddProcessorExisting()
        {
            IFileProcessor processor = new Processor.LineCounter.TextLineCounter("cs", new Matcher.AnyFileMatcher());
            instances[1].AddFileProcessor(processor);
            Assert.AreEqual(2, instances[1].AllFileProcessors.Count, "Count shouldn't change.");
            Assert.AreEqual(processor, instances[1]["cs"], "The associated processor should have changed.");
        }

        /// <summary>
        /// Tests adding a null processor.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestAddProcessorNull()
        {
            instances[1].AddFileProcessor(null);
        }

        /// <summary>
        /// Tests clearing all processors.
        /// </summary>
        [Test]
        public void TestClearAllProcessors()
        {
            instances[1].ClearAllFileProcessors();
            Assert.AreEqual(0, instances[1].AllFileProcessors.Count, "No processors should now be present.");
        }

        /// <summary>
        /// Tests processing two files with both instances.
        /// </summary>
        [Test]
        public void TestProcessFile()
        {
            string[] filenames = new string[]
            {
                @"..\..\test_files\sub\TestFile1.cs",
                @"..\..\test_files\sub\exe\test.exe"
            };

            for (int i = 0; i < 2; i++)
            {
                for (int j = 0; j < 2; j++)
                {
                    instances[i].Reporting.Reset(null);
                    instances[i].ProcessFile(new FileInfo(filenames[j]));

                    Reporting.AbstractStatsReporting reporter =
                        (Reporting.AbstractStatsReporting) instances[i].Reporting;
                    if (i == 0)
                    {
                        Assert.AreEqual(0, reporter.OverallStats.Metrics.Count, "No metrics should be present.");
                    }
                    else
                    {
                        Assert.AreEqual(1, reporter.OverallStats.Metrics.Count, "One metric should be present.");
                        if (j == 0)
                        {
                            Assert.AreEqual(13, reporter.OverallStats[new Metric("Line Count", "a", "a")],
                                "Expected aggregated line count for C# source.");
                        }
                        else
                        {
                            Assert.AreEqual(204, reporter.OverallStats[new Metric("Line Count", "a", "a")],
                                "Expected 'text-only' line count for EXE.");
                        }
                    }
                }
            }
        }

        /// <summary>
        /// Tests <c>ProcessFile</c> with a null argument.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestProcessFileNull()
        {
            instances[1].ProcessFile(null);
        }
    }
}