/*
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER, TCSDESIGNER
 * Version: 1.0
 */

using System;
using NUnit.Framework;

namespace TopCoder.File.Statistics
{
    /// <summary>
    /// This class tests the <c>StatsResult</c> class.
    /// </summary>
    [TestFixture]
    public class TestStatsResult
    {
        /// <summary>
        /// Three metrics used for testing.
        /// </summary>
        private Metric[] metrics = new Metric[3];

        /// <summary>
        /// Three <c>StatsResult</c> instances used for testing.
        /// </summary>
        private StatsResult[] results = new StatsResult[3];

        /// <summary>
        /// Sets up some simple objects used for testing.
        /// </summary>
        [SetUp]
        public void Setup()
        {
            metrics[0] = new Metric("A", "B", "C");
            metrics[1] = new Metric("A", "C", "B");
            metrics[2] = new Metric("D", "B", "C");

            results[0] = new StatsResult();
            results[1] = new StatsResult();
            results[2] = new StatsResult();

            results[1][metrics[0]] = 1;
            // When we merge result 1 and 2, metrics 0 and 1 should be aggregated, as they share the name "A"
            results[2][metrics[1]] = 2;
            results[2][metrics[2]] = 4;
        }

        /// <summary>
        /// Tests the Metrics property, both Count and contents.
        /// </summary>
        [Test]
        public void TestMetrics()
        {
            for (int i = 0; i < 3; i++)
            {
                Assert.AreEqual(i, results[i].Metrics.Count, "Metric count should match.");
            }

            Assert.IsTrue(results[1].Metrics.Contains(metrics[1]), "Metric 1 should be present.");
            Assert.IsTrue(results[2].Metrics.Contains(metrics[0]), "Metric 0 should be present.");
            Assert.IsTrue(results[2].Metrics.Contains(metrics[2]), "Metric 2 should be present.");
        }

        /// <summary>
        /// Tests aggregation.
        /// </summary>
        [Test]
        public void TestAggregate()
        {
            results[1].Aggregate(results[2]);
            Assert.AreEqual(3, results[1][metrics[0]], "Results for metric 0/1 should be aggregated.");
            Assert.AreEqual(4, results[1][metrics[2]], "Results for metric 2 should just be added.");

            results[0].Aggregate(results[2]);
            Assert.AreEqual(2, results[0][metrics[0]], "Results for metric 0/1 should be added to empty results.");
            Assert.AreEqual(4, results[0][metrics[2]], "Results for metric 2 should be added to empty results.");
        }

        /// <summary>
        /// Tests null aggregation, which isn't allowed.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestAggregateNull()
        {
            results[0].Aggregate(null);
        }

        /// <summary>
        /// Tests the cloning performance.
        /// </summary>
        [Test]
        public void TestCloneConstructor()
        {
            // Clone result 2
            results[0] = new StatsResult(results[2]);
            Assert.AreEqual(2, results[0][metrics[0]], "Results for metric 0/1 should be added to empty results.");
            Assert.AreEqual(4, results[0][metrics[2]], "Results for metric 2 should be added to empty results.");
        }

        /// <summary>
        /// Tests a null parameter for the clone constructor.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestCloneConstructorNull()
        {
            new StatsResult(null);
        }

        /// <summary>
        /// Tests a null metric for the class indexer.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestIndexerNull()
        {
            long x = results[0][null];
        }

        /// <summary>
        /// Tests a non-existing metric for the class indexer.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestIndexerNotPresent()
        {
            long x = results[0][metrics[0]];
        }

        /// <summary>
        /// Tests retrieving values by the means of the class indexer.
        /// </summary>
        [Test]
        public void TestIndexer()
        {
            Assert.AreEqual(1, results[1][metrics[1]], "Results 1, metric 1, retrieved incorrectly.");
            Assert.AreEqual(2, results[2][metrics[0]], "Results 2, metric 0, retrieved incorrectly.");
            Assert.AreEqual(4, results[2][metrics[2]], "Results 2, metric 2, retrieved incorrectly.");
        }
    }
}