/*
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER, TCSDESIGNER
 * Version: 2.0b
 */


using System;
using TopCoder.Util.DataValidator;

namespace TopCoder.File.Statistics
{
    /// <summary>
    /// An <c>ArgumentOption</c> is an immutable data storage class to communicate available options through the
    /// <c>IArgumentHandler interface</c>. It details the switch name of the option, whether the option is required,
    /// an optional data validator object and a help message.
    /// </summary>
    /// <remarks>
    /// This class is immutable and thread-safe.
    /// </remarks>
    public class ArgumentOption
    {
        /// <summary>
        /// The name of the command-line switch.
        /// </summary>
        private readonly string switchName;

        /// <summary>
        /// A help message shown to the user for this switch. If multiple plugins provide the same switch, only one is
        /// shown, but it isn't defined which one.
        /// </summary>
        private readonly string helpMessage;

        /// <summary>
        /// Determines if the switch is required, i.e. if a command line without this switch included is not to be
        /// considered valid.
        /// </summary>
        private readonly bool required;

        /// <summary>
        /// An <c>IValidator</c> implementation to check the argument string to the switch. Null allowed, then all
        /// validation must be performed by the argument handler on its own.
        /// </summary>
        private readonly IValidator validator;

        /// <summary>
        /// Exposes the switch name for this argument option. The switch name shouldn't include "/" or "-", those are
        /// added by the command line parser. (Getter only.)
        /// </summary>
        /// <value>
        /// A non-null, non-empty string with no spaces. More specific info can be found in the command line parser
        /// component docs.
        /// </value>
        public string SwitchName
        {
            get
            {
                return switchName;
            }
        }

        /// <summary>
        /// A help message shown to the user for this switch. If multiple plugins provide the same switch, only one is
        /// shown, but it isn't defined which one. (Getter only.)
        /// </summary>
        /// <value>
        /// A non-null string, non-empty string.
        /// </value>
        public string HelpMessage
        {
            get
            {
                return helpMessage;
            }
        }

        /// <summary>
        /// Determines if the switch is required, i.e. if a command line without this switch included is not to be
        /// considered valid. (Getter only.)
        /// </summary>
        /// <value>
        /// <c>bool</c>
        /// </value>
        public bool IsRequired
        {
            get
            {
                return required;
            }
        }

        /// <summary>
        /// An <c>IValidator</c> implementation to check the argument string to the switch. Null allowed, then all
        /// validation must be performed by the argument handler on its own.
        /// </summary>
        /// <value>
        /// <c>null</c> or a valid <c>IValidator</c> implementation that validates the argument strings.
        /// </value>
        public IValidator Validator
        {
            get
            {
                return validator;
            }
        }

        /// <summary>
        /// A three-argument constructor, implicit null validator.
        /// </summary>
        /// <param name="switchName">The switch name.</param>
        /// <param name="helpMessage">The help message.</param>
        /// <param name="required">Determines if the option is to be required.</param>
        /// <exception cref="ArgumentNullException">If <c>switchName</c> or <c>helpMessage</c> is null.</exception>
        /// <exception cref="ArgumentException">If <c>switchName</c> or <c>helpMessage</c> is empty when trimmed.
        /// </exception>
        public ArgumentOption(string switchName, string helpMessage, bool required) :
            this(switchName, helpMessage, required, null)
        {
        }

        /// <summary>
        /// A four-argument constructor for all the data values of the class.
        /// </summary>
        /// <param name="switchName">The switch name.</param>
        /// <param name="helpMessage">The help message.</param>
        /// <param name="required">Determines if the option is to be required.</param>
        /// <param name="validator">An <c>IValidator</c> to use for argument strings.</param>
        /// <exception cref="ArgumentNullException">If <c>switchName</c> or <c>helpMessage</c> is null.</exception>
        /// <exception cref="ArgumentException">If <c>switchName</c> or <c>helpMessage</c> is empty when trimmed.
        /// </exception>
        public ArgumentOption(string switchName, string helpMessage, bool required, IValidator validator)
        {
            this.switchName = switchName;
            this.helpMessage = helpMessage;
            this.required = required;
            this.validator = validator;

            if (switchName == null)
            {
                throw new ArgumentNullException("switchName");
            }
            if (switchName.Trim().Length == 0)
            {
                throw new ArgumentException("Empty switch name specified.", "switchName");
            }

            if (helpMessage == null)
            {
                throw new ArgumentNullException("helpMessage");
            }
            if (helpMessage.Trim().Length == 0)
            {
                throw new ArgumentException("Empty help message specified.", "helpMessage");
            }
        }
    }
}
