/*
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER, TCSDESIGNER
 * Version: 2.0b
 */

using System; 
using System.IO;
using TopCoder.Util.CommandLine;
using TopCoder.Util.ConfigurationManager;
using System.Collections;
using System.Text;

namespace TopCoder.File.Statistics
{
    /// <summary>
    /// This class is a command line interface to the component.
    /// </summary>
    /// <remarks>
    /// The following options are supported:
    /// <list>
    /// <item>
    /// /c:config-file
    /// A config file that's initialized (otherwise based on preload for configuration manager).
    /// </item>
    /// <item>
    /// /n:namespace Provides a namespace to initialize from. If neither is provided the default namespace is used.
    /// That namespace should be preloaded.
    /// </item>
    /// <item>
    /// /b:base-path
    /// All paths are relative to this base.
    /// </item>
    /// <item>
    /// /r
    /// Enables recursive descent through sub-directories.
    /// </item>
    /// <item>
    /// /o:output-file
    /// An output filename. If not specified, output goes to console.
    /// </item>
    /// </list>
    /// </remarks>
    public class CommandLine
    {
        /// <summary>
        /// This is the default namespace used if none is specifically defined in the command line arguments.
        /// </summary>
        public const string Namespace = "TopCoder.File.Statistics.CommandLine";

        /// <summary>
        /// The help message that's shown if the command is called without options or with invalid options.
        /// Plugin options are switched between the two array elements.
        /// </summary>
        private static readonly string[] HelpMessage = new string[] {
            @"Generates a file statistics report.

Usage: {0} [/c:config-file] [/n:namespace] [/b:base-path] [/r] [/o] {1}path [path2 path3 ...]

  /c:config-file    A config file to load.
  /n:namespace      Provides a namespace to initialize from. If neither of
                    these are provided the default namespace is used. That
                    namespace should be preloaded.
  /b:base-path      All paths are relative to this base.
  /r                Enabls recursive descent through sub-directories.
  /o:output-file    An output filename. If not specified, output goes to
                    console.",
            @"  paths             The paths to survey.

"};

        /// <summary>
        /// The number of character positions between the "/" and the help message in a command switch help line.
        /// </summary>
        private const int HelpSpaces = 17;

        /// <summary>
        /// A constructor to stop from instantiation of this class, as that isn't intended.
        /// </summary>
        private CommandLine()
        {
        }

        /// <summary>
        /// Prints the help message, with the correct file name inserted.
        /// </summary>
        public static void PrintHelp(IDictionary supportedOptions)
        {
            StringBuilder pluginSwitchString = new StringBuilder();
            foreach (DictionaryEntry entry in supportedOptions)
            {
                ArgumentOption argument = (ArgumentOption) entry.Value;
                if (!argument.IsRequired)
                {
                    pluginSwitchString.Append("[");
                }

                pluginSwitchString.Append("/");
                pluginSwitchString.Append(argument.SwitchName);

                if (!argument.IsRequired)
                {
                    pluginSwitchString.Append("]");
                }

                pluginSwitchString.Append(" ");
            }

            Console.Error.WriteLine(HelpMessage[0], typeof(CommandLine).Assembly.Location,
                                    pluginSwitchString.ToString());
            foreach (DictionaryEntry entry in supportedOptions)
            {
                ArgumentOption argument = (ArgumentOption) entry.Value;
                pluginSwitchString.Remove(0, pluginSwitchString.Length);

                int spacing = HelpSpaces - argument.SwitchName.Length;

                // Have at least one space, no matter what the length dictated.
                if (spacing < 1)
                {
                    spacing = 1;
                }

                pluginSwitchString.Append(' ', spacing);
                Console.Error.WriteLine("  /{0}{1}{2}", argument.SwitchName, pluginSwitchString.ToString(),
                                        argument.HelpMessage);
            }
            Console.Error.WriteLine(HelpMessage[1]);
        }

        /// <summary>
        /// A helper to parse plugin switches, validate them and hand them over to the plugins.
        /// </summary>
        /// <param name="args">The command line arguments.</param>
        /// <param name="statEngine">The <c>FileStatistics</c> implementation to call.</param>
        /// <param name="supportedOptions">A dictionary where to store all supported options for creating help messages.
        /// </param>
        /// <exception cref="Exception">Basically anything, we aren't too careful as we expect it to be picked up
        /// outside.</exception>
        private static void ParseOptions(string[] args, FileStatistics statEngine, IDictionary supportedOptions)
        {
            IList argumentHandlers = statEngine.GetAllArgumentHandlers();
            IDictionary argumentHandlersMap = new Hashtable();

            foreach (IArgumentHandler handler in argumentHandlers)
            {
                IList list = handler.ProposeOptions();
                argumentHandlersMap[handler] = list;

                // "Merge" requirements and validators.
                foreach (ArgumentOption argument in list)
                {
                    supportedOptions[argument.SwitchName] = argument;
                }
            }

            // Build a list for the command line parser.
            ArrayList finalOptions = new ArrayList(supportedOptions.Count);
            foreach (DictionaryEntry entry in supportedOptions)
            {
                CommandLineSwitch option = new CommandLineSwitch((string) entry.Key);
                // BUG! (not in this component, command line parser)
                option.Factor = (int) FactorParams.ONE_OR_MORE;
                finalOptions.Add(option);
            }

            CommandLineParser parser = new CommandLineParser(finalOptions.ToArray(typeof(CommandLineSwitch)), false);
            parser.Parse(args);

            // Inform the plugins about the results
            foreach (IArgumentHandler handler in argumentHandlers)
            {
                IDictionary arguments = new Hashtable();

                foreach (ArgumentOption argument in (IList) argumentHandlersMap[handler])
                {
                    if (parser.InputtedSwitches.Contains(argument.SwitchName))
                    {
                        string value = "";
                        string[] result = (string[])
                            ((CommandLineSwitch) parser.InputtedSwitches[argument.SwitchName]).Result;
                        if (result.Length != 1)
                        {
                            value = result[0];
                        }
                        if (result.Length > 1)
                        {
                            throw new ArgumentException("Multi-valued plugin switches not supported.");
                        }

                        if (argument.Validator != null && !argument.Validator.IsValid(value))
                        {
                            throw new InvalidArgumentException("Switch " + argument.SwitchName + " for " + handler +
                                                        "didn't pass validation.");
                        }

                        arguments[argument.SwitchName] = value;
                    }
                    else
                    {
                        if (argument.IsRequired)
                        {
                            // A missing required argument is rather equivalent to this condition.
                            throw new MissingInitializationException("Required switch " + argument.SwitchName + " for "
                                                                     + handler + "missing.");
                        }
                    }
                }

                handler.HandleArguments(arguments);
            }
        }

        /// <summary>
        /// Command line entrypoint. The class documentation details the available options.
        /// </summary>
        /// <param name="args"></param>
        public static void Main(string[] args)
        {
            // This is the statistics engine object. It isn't actually set until the switches have been parsed.
            ExtendedFileStatistics statEngine;

            // This is the list of supported options. We start at empty, depending on when an error happens and we want
            // to display the help message, we may not have access to a relevant list of supported options.
            IDictionary supportedOptions = new Hashtable();

            // If no arguments, print help and quit
            if (args.Length == 0)
            {
                // We try to get a list of plugins, but it won't be accurate for a specific configuration.
                statEngine = new ExtendedFileStatistics();

                try
                {
                    ParseOptions(args, statEngine, supportedOptions);
                }
                catch (Exception)
                {
                    // We WILL probably have exceptions, but it's obvious when the command line is empty.
                }

                PrintHelp(supportedOptions);
                return;
            }

            // Prepare the config fields.
            // They are not thread-safe, we must be careful and not share them if this class should be called
            // programmatically.
            CommandLineSwitch configFileOption = new CommandLineSwitch("c");
            configFileOption.Factor = (int) FactorParams.ONE_OR_MORE;

            CommandLineSwitch namespaceOption = new CommandLineSwitch("n");
            configFileOption.Factor = 1;

            CommandLineSwitch basePathOption = new CommandLineSwitch("b");
            basePathOption.Factor = 1;

            CommandLineSwitch recursiveOption = new CommandLineSwitch("r");
            // This is really a bug in the command line parser. Fix when that's fixed.
            recursiveOption.Factor = (int) FactorParams.ZERO_OR_MORE;

            CommandLineSwitch outputFileOption = new CommandLineSwitch("o");
            outputFileOption.Factor = 1;

            CommandLineParser parser = new CommandLineParser(new CommandLineSwitch[]
                {
                    configFileOption,
                    namespaceOption,
                    basePathOption,
                    recursiveOption,
                    outputFileOption
                }, false);

            // Parsing and some reading is done here. Error in this part results in a help message.
            // We need to parse the arguments once, just to be able to know what configuration to load, to be able to
            // determine what other switches we should support.
            string basePath = "";
            try
            {
                parser.Parse(args);

                if (configFileOption.Result != null)
                {
                    foreach (string configFile in configFileOption.Result)
                    {
                        ConfigManager.GetInstance().LoadFile(configFile);
                    }
                }

                string nameSpace = Namespace;
                if (namespaceOption.Result != null)
                {
                    nameSpace = (string) namespaceOption.Result[0];
                }

                statEngine = new ExtendedFileStatistics(nameSpace);
                ParseOptions(args, statEngine, supportedOptions);

                if (recursiveOption.Result != null && recursiveOption.Result.Length != 0)
                {
                	throw new ArgumentException("Recursive switch takes no arguments.");
                }

                if (basePathOption.Result != null)
                {
                    statEngine.Reporting.Reset(new DirectoryInfo((string) basePathOption.Result[0]));
                    basePath = (string) basePathOption.Result[0];
                }

                if (parser.Arguments.Length == 0)
                {
                    throw new ArgumentException("No paths to survey specified.");
                }
            }
            catch (Exception e)
            {
                Console.Error.WriteLine(e);
                Console.Error.WriteLine();
                PrintHelp(supportedOptions);
                return;
            }

            // These are errors while running, help doesn't make much sense.
            try
            {
                bool recursive = (parser.InputtedSwitches["r"] != null);
                foreach (string str in parser.Arguments)
                {
                    string realStr = Path.Combine(basePath, str);

                    FileInfo file = new FileInfo(realStr);
                    if (file.Exists)
                    {
                        statEngine.ProcessFile(file);
                    }
                    else
                    {
                        DirectoryInfo directory = new DirectoryInfo(realStr);
                        statEngine.ProcessDirectory(directory, recursive);
                    }
                }

                if (outputFileOption.Result != null)
                {
                    statEngine.PrintReport(new FileInfo((string) outputFileOption.Result[0]));
                }
                else
                {
                    statEngine.PrintReport();
                }
            }
            catch (Exception e)
            {
                Console.Error.WriteLine(e);
                Console.Error.WriteLine();
            }
        }
    }
}
