/*
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER, TCSDESIGNER
 * Version: 2.0b
 */


using System;
using System.IO;

namespace TopCoder.File.Statistics.Matcher
{
    /// <summary>
    /// An abstract implementation of <c>IFileMatcher</c> that identifies if a file is binary or text. Subclasses
    /// are provided for simple configuration, based on class name.
    /// </summary>
    /// <remarks>
    /// No configuration is read by this class. The implementation is stateless and thread-safe. Subclasses may break
    /// either or both of these, but they probably won't have to. The subclasses provided in this component simply
    /// match all binary files, or all text files. <see cref="IsBinary" /> for a definition of what is considered a
    /// text file by the algorithm used.
    /// </remarks>
    public abstract class FileCategoryMatcher : IFileMatcher
    {
        /// <summary>
        /// This constant determines the number of bytes to consider when determining if a file is binary or not.
        /// Note that as the default value is small, 1 KB, a buffer of that size is allocated directly in the
        /// implementation. If this value would be raised to something like 1 MB or so, it would probably be wise to
        /// change the implementation to read smaller chunks.
        /// </summary>
        private const int BytesToConsider = 1024;

        /// <summary>
        /// Default constructor. Does nothing.
        /// </summary>
        protected FileCategoryMatcher()
        {
        }

        /// <summary>
        /// Helper method to determine if a specific file is binary or text.
        /// </summary>
        /// <remarks>
        /// The algorithm used will only examine the first 1024 bytes, no matter the real file size, to avoid a too
        /// high performance penalty. The algorithm itself is based on identifying characters in the range [0, 31],
        /// which, while uncommon in most text files, generally tend to occur in binary files of any length.
        ///
        /// In that range, the following characters are considered valid in text files:
        /// <list>
        /// <item>4 (end-of-file in some UNIX environments)</item>
        /// <item>10 (line-feed, UNIX newline character)</item>
        /// <item>12 (form-feed, used for some pre-paginated text files)</item>
        /// <item>13 (carriage return, newline character, Windows uses CRLF)</item>
        /// <item>26 ("Ctrl+Z", used as end of file in some files)</item>
        /// </list>
        ///
        /// The method is public static to encourage reuse by other matchers or other code that would need this
        /// functionality.
        /// </remarks>
        /// <param name="file">The file to consider.</param>
        /// <exception cref="ArgumentNullException">If <c>file</c> is null.</exception>
        /// <exception cref="IOException">If any IO operation fails.</exception>
        /// <returns></returns>
        public static bool IsBinary(FileInfo file)
        {
            if (file == null)
            {
                throw new ArgumentNullException("file");
            }

            // This stream will use its own buffering scheme. Still, allocating a private buffer and using that for the
            // check itself is often better.
            // The complete buffer is read, even if it's highly probable that a binary file will disclose its
            // "identity" long before the buffer size is reached. If BytesToConsider is raised considerably, another
            // strategy may be wise.
            using (Stream stream = file.OpenRead())
            {
                byte[] buffer = new byte[BytesToConsider];
                int size = stream.Read(buffer, 0, BytesToConsider);

                for (int i = 0; i < size; i++)
                {
                    // If it's in the possible range of "binary characters", check it.
                    if (buffer[i] < 32)
                    {
                        switch (buffer[i])
                        {
                            case 4: // UNIX-style EOF
                            case 10: // UNIX-style EOL \n
                            case 12: // Form-feed \f
                            case 13: // Carriage-return \r
                            case 26: // DOS-style EOF
                                break;

                            default: // A binary character, even one and this file is considered to be binary
                                return true;
                        }
                    }
                }
                // No forbidden character found, this is a text file
                return false;
            }
        }

        /// <summary>
        /// Extracts the file type and passes the call on to the specific name checking implementation of the subclass.
        /// </summary>
        /// <param name="file">The file to match.</param>
        /// <exception cref="ArgumentNullException">If <c>file</c> is null.</exception>
        /// <exception cref="FileMatchingException">Other error conditions.</exception>
        /// <exception cref="IOException">If any IO operation fails.</exception>
        /// <returns><c>true</c> if the file name matches, <c>false</c> otherwise.</returns>
        public bool Match(FileInfo file)
        {
            return Match(IsBinary(file));
        }

        /// <summary>
        /// Template method to match a file whether it is binary or not.
        /// </summary>
        /// <param name="isBinary">Parameter telling if the file is binary, true, or text, false.</param>
        /// <exception cref="FileMatchingException">Any error condition.</exception>
        /// <returns>True if the file is matched, false otherwise.</returns>
        protected abstract bool Match(bool isBinary);
    }
}