/*
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER, TCSDESIGNER
 * Version: 2.0b
 */


using System;

namespace TopCoder.File.Statistics
{
    /// <summary>
    /// A <c>Metric</c> represents a measurement of the stats result. A metric is identified by its name. Metrics with
    /// the same name will be aggregated. This class overrides <c>Equals()</c> and <c>GetHashCode()</c> so that it can
    /// be used as map keys.
    /// </summary>
    /// <remarks>
    /// This class is immutable and thread-safe.
    /// </remarks>
    public class Metric
    {
        /// <summary>
        /// The name of the metric. Acts as an identifier.
        /// </summary>
        /// <remarks>
        /// As the whole class is immutable, this value is readonly.
        /// </remarks>
        private readonly string name;

        /// <summary>
        /// The display name in the rendered result. If this is used in XML reporting the display name should be a
        /// well-formed element name.
        /// </summary>
        /// <remarks>
        /// As the whole class is immutable, this value is readonly.
        /// </remarks>
        private readonly string displayName;

        /// <summary>
        /// A description for the metric. This does not make a significance in this release.
        /// </summary>
        /// <remarks>
        /// As the whole class is immutable, this value is readonly.
        /// </remarks>
        private readonly string description;

        /// <summary>
        /// Exposes the name of the metric, only a getter.
        /// </summary>
        /// <value>
        /// <c>this.name</c>
        /// </value>
        public string Name
        {
            get
            {
                return name;
            }
        }

        /// <summary>
        /// Exposes the user-friendly display name, only a getter.
        /// </summary>
        /// <value>
        /// <c>this.displayName</c>
        /// </value>
        public string DisplayName
        {
            get
            {
                return displayName;
            }
        }

        /// <summary>
        /// Exposes the metric description, only a getter.
        /// </summary>
        /// <value>
        /// <c>this.description</c>
        /// </value>
        public string Description
        {
            get
            {
                return description;
            }
        }

        /// <summary>
        /// Constructor, requiring all arguments.
        /// </summary>
        /// <param name="name">The name of the metric.</param>
        /// <param name="displayName">The display name for the metric.</param>
        /// <param name="description">The description to use for the metric.</param>
        /// <exception cref="ArgumentNullException">If any parameter is null.</exception>
        /// <exception cref="ArgumentException">If any parameter, after trimming, is empty.</exception>
        public Metric(string name, string displayName, string description)
        {
            if (name == null)
            {
                throw new ArgumentNullException("name");
            }
            if (displayName == null)
            {
                throw new ArgumentNullException("name");
            }
            if (description == null)
            {
                throw new ArgumentNullException("description");
            }

            if (name.Trim().Length == 0)
            {
                throw new ArgumentException("Empty string specified.", "name");
            }
            if (displayName.Trim().Length == 0)
            {
                throw new ArgumentException("Empty string specified.", "displayName");
            }
            if (description.Trim().Length == 0)
            {
                throw new ArgumentException("Empty string specified.", "description");
            }

            this.name = name;
            this.displayName = displayName;
            this.description = description;
        }

        /// <summary>
        /// Implements equality.
        /// </summary>
        /// <param name="obj">The object to compare against.</param>
        /// <returns>True if and only if <c>obj</c> is also a <c>Metric</c> with a matching <c>Name</c>.</returns>
        public override bool Equals(object obj)
        {
            return (obj is Metric) && ((Metric) obj).Name == Name;
        }

        /// <summary>
        /// Hash codes for this class are directly mapped to corresponding values for the <c>Name</c> property.
        /// </summary>
        /// <returns>
        /// <c>Name.GetHashCode()</c>
        /// </returns>
        public override int GetHashCode()
        {
            return Name.GetHashCode();
        }
    }
}